             ********************************************
             ***'                                    '***
             ***'.........SER2 Testprogramm..........'***
             ***'                                    '***
             ***'  DUART  XR88C681CP28/CP40          '***
             ***'                                    '***
             ***'  (P)+(C) 2007  Gerald Ebert        '***
             ***'                                    '***
             ********************************************

*>>>>>>>    LABELS

cpu         equ   2                   * M68000/10

s2base      equ   $ffffff90           * Basis I/O-Adresse ($90-$9F)

s2mra       equ   (s2base+0)*cpu      * Mode Register Channel A
s2sra       equ   (s2base+1)*cpu      * Status Register Channel A
s2csra      equ   s2sra               * Clock Select Register Channel A
s2misr      equ   (s2base+2)*cpu      * Masked Interrupt Status Register
s2cra       equ   s2misr              * Command Register Channel A
s2rhra      equ   (s2base+3)*cpu      * Rx Holding Register Channel A
s2thra      equ   s2rhra              * Tx Holding Register Channel A
s2ipcr      equ   (s2base+4)*cpu      * Input Port Change Register
s2acr       equ   s2ipcr              * Auxilary Control Register
s2isr       equ   (s2base+5)*cpu      * Interrupt Status Register
s2imr       equ   s2isr               * Interrupt Mask Register
s2ctu       equ   (s2base+6)*cpu      * Counter/Timer Upper Byte Register
s2ctl       equ   (s2base+7)*cpu      * Counter/Timer Lower Byte Register
s2mrb       equ   (s2base+8)*cpu      * Mode Register Channel B
s2srb       equ   (s2base+9)*cpu      * Status Register Channel B
s2csrb      equ   s2srb               * Clock Select Register Channel B
s2res       equ   (s2base+10)*cpu     * Reserved
s2crb       equ   s2res               * Command Register Channel B
s2rhrb      equ   (s2base+11)*cpu     * Rx Holding Register Channel B
s2thrb      equ   s2rhrb              * Tx Holding Register Channel B
s2ivr       equ   (s2base+12)*cpu     * Interrupt Vector Register
s2ip        equ   (s2base+13)*cpu     * Input Port
s2opcr      equ   s2ip                * Output Port Configuration Register
s2sopb      equ   (s2base+14)*cpu     * Set Output Port Bits
s2copb      equ   (s2base+15)*cpu     * Clear Output Port Bits

*>>>>>>     Tabelle fr Baudrate-Auswahl

s2bd9600    equ   $bb
s2bd19k2    equ   $cc
s2bd28k8    equ   $66
s2bd57k6    equ   $77
s2bd115k    equ   $88

*>>>>>>     START DES TESTPROGRAMMS

            move.b   #$03, s2sopb.w     * LEDs ausschalten

;' Serielle Schnittstelle Channel A initialisieren
            move.b   #s2bd115k, d0      * 115 kBaud einstellen
            bsr      s2initA

;' Text auf serielle Schnittstelle Channel A ausgeben
            lea      text(pc), a0
loop1:      move.b   (a0)+, d0
            beq      next1
            bsr      s2sendA
            bra      loop1

;' Zeichen von serieller Schnittstelle holen und ausgeben
next1:      moveq    #!CSTS, d7         * Taste gedrueckt
            trap     #1
            tst      d0
            bne      next2              * Ja, Einlesen abbrechen
            bsr      s2readA
            bcs.s    next1              * kein Zeichen da, warten
            moveq    #!CO, d7           * Zeichen in d0 ausgeben
            trap     #1
            bra.s    next1

next2:      moveq    #!CI, d7           * Zeichen von Tastatur noch holen
            trap     #1
            rts                         * ENDE

text:       dc.b     'Hallo PC. Ich bin ein NDR-Klein-Computer.', 13, 10
            dc.b     0


*>>>>>>     GLOBALE RUECKSPRUNGROUTINEN

carryclr:   andi  #$fe, ccr
            rts
carryset:   ori   #$01, ccr
ende:       rts

*>>>>>>     WARTESCHLEIFE FUER INIT

wawa:       nop
            rts

*>>>>>>     INITIALISIERE CHANNEL A
;'          d0 => Baudrate (siehe Tabelle)

s2initA:    move.b   d0, s2csra.w         * Setze Baudrate
            bsr      wawa
            move.b   #$03, s2mra.w        * Even Parity, 8 bit Daten
            bsr      wawa
            move.b   #$07, s2mra.w        * Ein Stop bit
            bsr      wawa
            move.b   #$80, s2cra.w        * Setze Extent Bit fuer Rx Cha A
            bsr      wawa
            move.b   #$a0, s2cra.w        * Setze Extent Bit fuer Tx Cha A
            bsr      wawa
            move.b   #$15, s2cra.w        * Starte Channel A
            rts

*>>>>>>     SENDE EIN BYTE AUF CHANNEL A
;'          d0 => zu sendendes Byte

s2sendA:    movem.l  d1, -(a7)
            move.b   #$01, s2copb.w       * Channel A LED einschalten
            bsr      wawa
lbl1:       move.b   s2sra.w, d1          * Hole Channel A Status register
            btst.b   #2, d1               * Ausgaberegister leer
            beq.s    lbl1                 * Nein -> weiter warten
            move.b   d0, s2thra.w         * Byte ins Ausgaberegister
            bsr      wawa
            move.b   #$01, s2sopb.w       * Channel A LED ausschalten
            movem.l  (a7)+, d1
            rts

*>>>>>>     HOLE EIN BYTE VON CHANNEL A
;'          d0 <= empfangens Byte, wenn C = 0

s2readA:    move.b   s2sra.w, d0          * Hole Channel A Status Register
            btst.b   #0, d0               * Stehen Daten in Empfangsfifo
            beq      carryset             * Nein -> fifo leer -> C = 1
            move.b   s2rhra.w, d0         * Byte aus Empfangsfifo holen
            bra      carryclr             * Zeichen in d0 gueltig -> C = 0

*>>>>>>     INITIALISIERE CHANNEL B
;'          d0 => Baudrate (siehe Tabelle)

s2initB:    move.b   d0, s2csrb.w         * Setze Baudrate
            bsr      wawa
            move.b   #$03, s2mrb.w        * Even Parity, 8 bit Daten
            bsr      wawa
            move.b   #$07, s2mrb.w        * Ein Stop bit
            bsr      wawa
            move.b   #$80, s2crb.w        * Setze Extent Bit fuer Rx Cha B
            bsr      wawa
            move.b   #$a0, s2crb.w        * Setze Extent Bit fuer Tx Cha B
            bsr      wawa
            move.b   #$15, s2crb.w        * Starte Channel B
            rts

*>>>>>>     SENDE EIN BYTE AUF CHANNEL B
;'          d0 => zu sendendes Byte

s2sendB:    movem.l  d1, -(a7)
            move.b   #$02, s2copb.w       * Channel B LED einschalten
            bsr      wawa
lbl2:       move.b   s2srb.w, d1          * Hole Channel B Status register
            btst.b   #2, d1               * Ausgaberegister leer
            beq.s    lbl2                 * Nein -> weiter warten
            move.b   d0, s2thrb.w         * Byte ins Ausgaberegister
            bsr      wawa
            move.b   #$02, s2sopb.w       * Channel B LED ausschalten
            movem.l  (a7)+, d1
            rts

*>>>>>>     HOLE EIN BYTE VON CHANNEL B
;'          d0 <= empfangens Byte, wenn C = 0

s2readB:    move.b   s2srb.w, d0         * Hole Channel B Status Register
            btst.b   #0, d0              * Stehen Daten in Empfangsfifo
            beq      carryset            * Nein -> fifo leer -> C = 1
            move.b   s2rhrb.w, d0        * Byte aus Empfangsfifo holen
            bra      carryclr            * Zeichen in d0 gueltig -> C = 0

            end
